///File : RcManager.hpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined RC_MANAGER_HPP_MAXI_SHOOT
#define RC_MANAGER_HPP_MAXI_SHOOT
//------------------------------------------------------------------------------
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>

#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm> // Sort
//------------------------------------------------------------------------------
#include "myConf.hpp"
//------------------------------------------------------------------------------
// Str to nb / nb to str
std::string intToString(int nb);
int stringToInt(std::string nbStr);
float stringToFloat(std::string nbStr);
std::string floatToString(float nb);
//------------------------------------------------------------------------------
#define MAX_SIMULTANEOUS_SND 4

enum {LOWER, LOWER_EQUAL, EQUAL, UPPER_EQUAL, UPPER, DIFFERENT};

struct StructNeeds
{
    std::string varName;
    int compareType;
    std::string value;
};

struct StructStockEnemy
{
    std::map<std::string, std::string> map;
    std::vector<StructNeeds> needs;
};

struct StructEnemy
{
    sf::Sprite spr;
    std::string imageResistAlias;
    sf::Vector2f imageResistPos;
    std::string sndShieldShock;
    std::string sndBurst;
    std::string sndResist;
    std::string sndEnemyShot;
    sf::Vector2f speed;
    float life;
    float domages;
    std::string specialSkill;
    std::map<std::string, float> specialValues;
};

struct StructFileList
{
    std::string file;
    std::string alias;
};

struct StructAllInfo
{
    std::map<std::string, sf::Image> m_ImgMap;
    std::map<std::string, sf::Font> m_FontMap;
    std::map<std::string, sf::SoundBuffer> m_SndBufMap;

    std::map<std::string, std::string> m_ValueMap;

    sf::Music m_MusicAmbience;

    int m_loadProgress;

    std::vector<StructFileList> m_FilesList;
    std::vector<StructStockEnemy> m_EnemyStockList;

    std::string m_modeDir;

    bool m_musicOn;
    bool m_soundOn;
};

// Fns
void GetFileListFromNormalFile(StructAllInfo &m_i, const std::string &filename);
void GetEnemies(StructAllInfo &m_i, const std::string &filename);
void AddFile(StructAllInfo &m_i, const std::string &modeDir, const std::string &realName, const std::string &alias);
void LoadNewModeThread(void* UserData);
void GetInfo(StructAllInfo &m_i, const std::string &filename);

class RcManager
{
public:
    void LoadNewMode(std::string modeDir);
    float GetLoadProgress() const;

    bool PlaySnd(const std::string &name);
    sf::Image &GetImage(const std::string &name);
    bool ImageExists(const std::string &name);
    sf::Font &GetFont(const std::string &name);
    sf::SoundBuffer &GetSoundBuffer(const std::string &name);

    StructEnemy NewEnemy();
    StructEnemy GetEnemyFromId(const std::string &id);

    void SetMusicOn(const bool &musicOn);
    bool IsMusicOn();

    void SetSoundOn(const bool &soundOn);
    bool IsSoundOn();

    void SetValue(const std::string &id,const float &value);
    void SetValue(const std::string &id,const std::string &value);
    bool ValueExists(const std::string &id);
    float GetValue(const std::string &id);
    std::string GetValueStr(const std::string &id);

    void NewGame();
private:
    StructEnemy CreateStructEnemy(const int &enemyId);
    bool CanUseEnemy(const int &enemyId);

    //void ReloadBackground();

    std::string Calcl(std::string value);
    std::string ParseStringInfo(std::string value);

    sf::Sound m_Snd[MAX_SIMULTANEOUS_SND];

    StructAllInfo m_i;
};
//------------------------------------------------------------------------------

#endif // RC_MANAGER_HPP_MAXI_SHOOT
