///File : main.cpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "main.hpp"
#ifdef __WXMAC__
    #include <ApplicationServices/ApplicationServices.h>
#endif
//------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    #ifdef __WXMAC__
    ProcessSerialNumber PSN;
    GetCurrentProcess(&PSN);
    TransformProcessType(&PSN,kProcessTransformToForegroundApplication);
    #endif
	
    #if USE_DEBUG_FILE
        // debug
        #if defined WIN32
        std::string debugFilename("C:\\maxishoot_test.txt");
        #else
        std::string debugFilename("/home/congelli501/Bureau/maxishoot_test.txt");
        #endif
        g_file_debug_flux = new std::ofstream(debugFilename.c_str(), std::ios::out | std::ios::trunc);  // on ouvre en ecriture
    #endif // USE_DEBUG_FILE

    // Config
    ConfigObj configObj;
    configObj.LoadModeDirNamesList();
    configObj.LoadConfig();

    sf::VideoMode videoMode = configObj.GetCurrentVideoMode();

    unsigned long windowStyle = 0;
    if(configObj.IsFullScreenOn())
    {
        windowStyle = sf::Style::Fullscreen;
    }
    else
    {
        windowStyle = sf::Style::Resize | sf::Style::Close;
    }

    // Create the main rendering window
    sf::RenderWindow App(videoMode, "Maxi Shoot", windowStyle);

    //Mouse cursor no more visible
    App.ShowMouseCursor(false);

    std::vector<Screen*> Screens;
    int screen = 0;

    Screens.push_back (new SplashScreen(&configObj));
    Screens.push_back (new MenuScreen(&configObj));
    Screens.push_back (new GameScreen(&configObj));
    Screens.push_back (new AboutScreen(&configObj));
    Screens.push_back (new LoadScreen(&configObj));

    while (screen >= 0 && screen < Screens.size())
    {
        screen = Screens[screen]->Run(App);
    }

    configObj.SaveAll();

    #if USE_DEBUG_FILE
        // debug
        g_file_debug_flux->close();
        delete g_file_debug_flux;
    #endif //USE_DEBUG_FILE

    exit(EXIT_SUCCESS);
}

